<?php
/**
 * Gestion du formulaire d'édition d'une historisation de Rainette.
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Chargement du formulaire d'édition d'une historisation (création ou modification).
 * Déclarer les champs postés et y intégrer les valeurs par défaut.
 *
 * @param null|string $histo_id Identifiant de l'historisation ou 'new' pour la création d'une nouvelle.
 * @param null|string $redirect URL de base du privé vers où se rendre en sortant du formulaire
 *
 * @return array Environnement du formulaire.
 */
function formulaires_editer_historisation_charger_dist(?string $histo_id = 'new', ?string $redirect = 'historisations') : array {
	// Initialisations des variables d'environnement du formulaire
	$valeurs = [
		'nom'           => '',
		'mode'          => 'conditions',
		'service'       => '',
		'lieu_id'       => '',
		'var_job'       => '',
		'active'        => true,
		'_modification' => '',
		'editable'      => false
	];

	if (
		(
			($histo_id === 'new')
			and autoriser('creer', 'historisation')
		)
		or (
			($histo_id !== 'new')
			and autoriser('modifier', 'historisation', $histo_id)
		)
	) {
		// Initialisations des modes possibles
		include_spip('inc/config');
		$config_rainette = lire_config('rainette_config', []);
		// -- Liste des modes autorisés
		foreach ($config_rainette['historiques']['modes'] as $_mode) {
			$valeurs['_modes'][$_mode] = _T('rainette:mode_' . $_mode);
		}

		// Initialiser les valeurs liées à l'historisation si celle-ci existe déjà
		if (
			($histo_id !== 'new')
			and ($historisation = lire_config("rainette_historisations/{$histo_id}", []))
		) {
			// Initialisation des saisies
			$valeurs = array_merge($valeurs, $historisation);

			// Disable des saisies nom et mode qui sont des invariants de l'historisation
			$valeurs['_modification'] = 'oui';
		}

		// Disable des saisies nom et mode qui sont des invariants de l'historisation
		$valeurs['editable'] = true;
	}

	return $valeurs;
}

/**
 * Vérifications du formulaire d'édition d'une historisation.
 * Vérifier les champs postés et signaler d'éventuelles erreurs.
 *
 * @param null|string $histo_id Identifiant de l'historisation ou 'new' pour la création d'une nouvelle.
 * @param null|string $redirect URL de base du privé vers où se rendre en sortant du formulaire
 *
 * @return array Tableau des erreurs indexé par le nom du champ en erreur.
 */
function formulaires_editer_historisation_verifier_dist(?string $histo_id = 'new', ?string $redirect = 'historisations') : array {
	$erreurs = [];

	// Vérifier les saisies obligatoires
	foreach (['mode', 'service', 'lieu_id'] as $_variable) {
		if (!_request($_variable)) {
			$erreurs[$_variable] = _T('info_obligatoire');
		}
	}

	// Si une erreur a été détectée on sort sinon on continue à en chercher
	if (!$erreurs) {
		// Acquisition et normalisation des saisies
		$mode = _request('mode');
		// -- on normalise le lieu
		include_spip('inc/rainette_normaliser');
		$lieu_normalise = lieu_normaliser(_request('lieu_id'));
		// -- on normalise un peu le nom
		if (!$nom = strtolower(trim(_request('nom')))) {
			$nom = "{$lieu_normalise['lieu_id']}/{$mode}";
		}

		// On vérifie que l'historisation n'est pas un doublon (nom et mode, ou lieu et mode) de façon non déterministe
		if (
			!_request('_modification')
			and ($historisations = lire_config('rainette_historisations', []))
		) {
			foreach ($historisations as $_histo_id => $_historisation) {
				if ($_historisation['mode'] === $mode) {
					if ($_historisation['lieu_id'] === $lieu_normalise['id']) {
						$erreurs['lieu_id'] = _T('rainette:erreur_histo_existe', ['histo_id' => $_histo_id]);
					} elseif (strtolower($_historisation['nom']) === $nom) {
						$erreurs['nom'] = _T('rainette:erreur_histo_existe', ['histo_id' => $_histo_id]);
					}
				}
			}
		}

		// Si une erreur a été détectée on sort sinon on continue à en chercher
		if (!$erreurs) {
			// On vérifie que le format du lieu est compatible avec le service
			$service = _request('service');
			if (!lieu_format_verifier_compatibilite($lieu_normalise['format'], $service)) {
				$erreurs['lieu_id'] = _T('rainette:erreur_lieu_format');
			}
		}
	}

	return $erreurs;
}

/**
 * Traitement du formulaire d'édition d'une historisation.
 * Traiter les champs postés.
 *
 * @param null|string $histo_id Identifiant de l'historisation ou 'new' pour la création d'une nouvelle.
 * @param null|string $redirect URL de base du privé vers où se rendre en sortant du formulaire
 *
 * @return array Retours des traitements.
 */
function formulaires_editer_historisation_traiter_dist(?string $histo_id = 'new', ?string $redirect = 'historisations') : array {
	$retour = [];

	// Récupération des saisies
	$historisation = [
		'nom'     => _request('nom'),
		'mode'    => _request('mode'),
		'service' => _request('service'),
		'lieu_id' => _request('lieu_id'),
		'via_job' => _request('via_job'),
	];

	// Initialisation de l'historisation à créer ou à modifier
	include_spip('inc/config');
	$parametres_editables = ['service', 'lieu_id', 'via_job'];
	// Date de l'action qui deviendra la date du log et aussi de la création si c'est le cas
	$date = date('Y-m-d H:i:s', time());
	if (
		($histo_id !== 'new')
		and ($historisation_existante = lire_config("rainette_historisations/{$histo_id}", []))
	) {
		// Modification de certains paramètres
		$historisation = array_merge($historisation_existante, $historisation);

		// Définition du log de modification
		$item_log = 'rainette:log_histo_modification';
		$parametres = [];
		foreach ($parametres_editables as $_parametre) {
			$parametres[$_parametre] = $historisation[$_parametre];
		}
	} else {
		// Création d'une nouvelle historisation
		// -- Création d'un unique id
		$histo_id = uniqid('h', true);

		// -- Normalisation du lieu et extraction de son id
		include_spip('inc/rainette_normaliser');
		$lieu_normalise = lieu_normaliser($historisation['lieu_id']);
		$historisation['lieu_id'] = $lieu_normalise['id'];

		// -- Création d'un nom d'historisation si celui-ci a été laissé vide
		if (!$historisation['nom']) {
			$historisation['nom'] = "{$historisation['lieu_id']}/{$historisation['mode']}";
		}

		// -- Activité de l'historisation : par défaut, est inactive à la création
		$historisation['active'] = false;

		// -- Ajout de la date de création pour éviter d'aller la chercher dans le log
		$historisation['date_crea'] = $date;

		// -- Définition du log de création
		$item_log = 'rainette:log_histo_creation';
		$parametres = array_intersect_key($historisation, array_flip($parametres_editables));
	}

	// Finaliser le log
	// -- parametres modifiés
	$parametres_log = '';
	foreach ($parametres as $_parametre => $_valeur) {
		$parametres_log .= ($parametres_log ? ', ' : '') . $_parametre . '=' . $_valeur;
	}
	$historisation['log_edition'][$date] = _T($item_log, ['parametres' => $parametres_log]);

	// Mettre à jour ou créer l'historisation
	ecrire_config("rainette_historisations/{$histo_id}", $historisation);

	// Finalisation du formulaire et redirection
	$retour['redirect'] = generer_url_ecrire($redirect);
	$retour['editable'] = true;

	return $retour;
}
